@@ -6,9 +6,44 @@ from ..utils.cache import simple_cache
 from ..utils.ingredients import *
 from ..constants import *
 from .shared_import import *
+from pathlib import Path
 from PIL import Image, ImageDraw, ImageFont
+from model_resolver.cli import main as model_resolver_main
 import requests
 
+# Generate high res simple case no border
+def load_simple_case_no_border(high_res: bool) -> Image.Image:
+	path = f"{TEMPLATES_PATH}/simple_case_no_border.png"
+	img = Image.open(path)
+	if not high_res:
+		return img
+
+	# Make the image bigger on the right
+	middle_x = img.size[0] // 2
+	result = Image.new("RGBA", (img.size[0] + 1, img.size[1]))
+	result.paste(img, (0, 0))
+	img = img.crop((middle_x, 0, img.size[0], img.size[1]))
+	result.paste(img, (middle_x + 1, 0))
+	return result
+
+# Careful resize
+def careful_resize(image: Image.Image, max_result_size: int) -> Image.Image:
+	""" Resize an image while keeping the aspect ratio.\n
+	Example 1: if the image is 100x200 and the max_result_size is 150, the result will be 75x150.\n
+	Example 2: if the image is 200x100 and the max_result_size is 256, the result will be 256x128.\n
+	Args:
+		image			(Image):	The image to resize
+		max_result_size	(int):		The minimum result size of the image (ex: 256)
+	Returns:
+		Image: The resized image
+	"""
+	if image.size[0] >= image.size[1]:
+		factor = max_result_size / image.size[0]
+		return image.resize((max_result_size, int(image.size[1] * factor)), Image.NEAREST)
+	else:
+		factor = max_result_size / image.size[1]
+		return image.resize((int(image.size[0] * factor), max_result_size), Image.NEAREST)
+
 # Generate a border for a given Image
 def add_border(image: Image.Image, border_color: tuple, border_size: int, is_rectangle_shape: bool) -> Image.Image:
 	""" Add a border to every part of the image
@@ -59,7 +94,7 @@ def add_border(image: Image.Image, border_color: tuple, border_size: int, is_rec
 	return image
 
 # Generate an image showing the result count
-def image_count(count: int) -> Image:
+def image_count(count: int) -> Image.Image:
 	""" Generate an image showing the result count
 	Args:
 		count (int): The count to show
@@ -74,9 +109,9 @@ def image_count(count: int) -> Image:
 
 	# Calculate text size and positions of the two texts
 	text_width = draw.textlength(str(count), font = font)
-	text_height = font_size
-	pos_1 = (32-text_width), (32-text_height)
-	pos_2 = (30-text_width), (30-text_height)
+	text_height = font_size + 4
+	pos_1 = (34-text_width), (32-text_height)
+	pos_2 = (32-text_width), (30-text_height)
 	
 	# Draw the count
 	draw.text(pos_1, str(count), (50, 50, 50), font = font)
@@ -87,6 +122,7 @@ def image_count(count: int) -> Image:
 def generate_all_iso_renders(config: dict):
 	path = config['manual_path'] + "/items"
 	os.makedirs(f"{path}/{config['namespace']}", exist_ok = True)
+	for_model_resolver = {}
 	for item, data in config['database'].items():
 		
 		# If it's not a block, simply copy the texture
@@ -96,47 +132,28 @@ def generate_all_iso_renders(config: dict):
 			if not os.path.exists(f"{path}/{config['namespace']}/{item}.png") or not config['cache_manual_assets']:
 				super_copy(f"{config['textures_folder']}/{item}.png", f"{path}/{config['namespace']}/{item}.png")
 		except:
-			# Else, render all the block textures and faces
-			try:
-				# Skip if item is already generated (to prevent launcher OpenGL for nothing)
-				if os.path.exists(f"{path}/{config['namespace']}/{item}.png") and config['cache_manual_assets']:
-					continue
+			# Else, add the block to the model resolver list
+			# Skip if item is already generated (to prevent launcher OpenGL for nothing)
+			if os.path.exists(f"{path}/{config['namespace']}/{item}.png") and config['cache_manual_assets']:
+				continue
 
-				# # Load front texture
-				# sides = ("_front", "_side", "_top", "_bottom", "")
-				# front_path = f"{config['textures_folder']}/{item}"
-				# for side in sides:
-				# 	if os.path.exists(f"{front_path}{side}.png"):
-				# 		front_path += side
-				# 		break
-				# front_texture = Image.open(front_path + ".png")
-				# side_texture = front_texture
-				# top_texture = front_texture
+			# Add to the model resolver queue
+			rp_path = f"{config['namespace']}:block/{item}"
+			dst_path = f"{path}/{config['namespace']}/{item}.png"
+			for_model_resolver[rp_path] = dst_path
 
-				# # Try to load side
-				# side_path = f"{config['textures_folder']}/{item}_side.png"
-				# if os.path.exists(side_path):
-				# 	side_texture = Image.open(side_path)
-				
-				# # Try to load top texture
-				# top_path = f"{config['textures_folder']}/{item}_top.png"
-				# if os.path.exists(top_path):
-				# 	top_texture = Image.open(top_path)
-				
-				# # Make front texture 50% darker and side texture 25% darker
-				# front_texture = ImageEnhance.Brightness(front_texture).enhance(0.5)
-				# side_texture = ImageEnhance.Brightness(side_texture).enhance(0.75)
-
-				# # Render block and take a screenshot
-				# opengl.render_block(front_texture, side_texture, top_texture)
-				# opengl.take_screenshot(f"{path}/{config['namespace']}/{item}.png")
-				error("TODO")
-
-			except:
-				try:
-					super_copy(f"{config['textures_folder']}/{item}.png", f"{path}/{config['namespace']}/{item}.png")
-				except:
-					error(f"Failed to render iso for item '{item}', please add it manually to '{path}/{config['namespace']}/{item}.png'")
+	# Launch model resolvers for remaining blocks
+	if len(for_model_resolver) > 0:
+		load_dir = Path(config['build_resource_pack'])
+		debug("Generating iso renders for %d items" % len(for_model_resolver))
+		model_resolver_main(
+			render_size = config['opengl_resolution'],
+			load_dir = load_dir,
+			output_dir = None,
+			use_cache = False,
+			minecraft_version = "latest",
+			__special_filter__ = for_model_resolver
+		)
 	debug("Generated iso renders for all items, or used cached renders")
 
 	## Copy every used vanilla items
@@ -172,7 +189,6 @@ def generate_all_iso_renders(config: dict):
 			if response.status_code == 200:
 				with super_open(destination, "wb") as file:
 					file.write(response.content)
-				debug(f"Downloaded texture for item '{item}'!")
 			else:
 				warning(f"Failed to download texture for item '{item}', please add it manually to '{destination}'")
 				warning(f"Suggestion link: '{DOWNLOAD_VANILLA_ASSETS_SOURCE}'")
@@ -207,24 +223,21 @@ def generate_page_font(config: dict, name: str, page_font: str, craft: dict|None
 		image_path = f"{config['manual_path']}/items/{result_id}.png"
 		result_texture = Image.open(image_path)
 
-	# Resize the texture and get the mask
-	factor = SQUARE_SIZE / result_texture.size[0]
-	result_texture = result_texture.resize(
-		(int(result_texture.size[0]*factor), int(result_texture.size[1]*factor)),
-		Image.NEAREST
-	)
-	result_mask = result_texture.convert("RGBA").split()[3]
-	
 	# Check if there is a craft
 	if craft:
 
+		# Resize the texture and get the mask
+		result_texture = careful_resize(result_texture, SQUARE_SIZE)
+		result_mask = result_texture.convert("RGBA").split()[3]
+
 		# Shaped craft
 		if craft["type"] in "crafting_shaped":
 
 			# Get the image template and append the provider
 			shaped_size = max(2, max(len(craft["shape"]), len(craft["shape"][0])))
 			template = Image.open(f"{TEMPLATES_PATH}/shaped_{shaped_size}x{shaped_size}.png")
-			font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/page/{output_filename}.png", "ascent": 0 if not output_name else 6, "height": 60, "chars": [page_font]})
+			if not config['manual_high_resolution']:
+				font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/page/{output_filename}.png", "ascent": 0 if not output_name else 6, "height": 60, "chars": [page_font]})
 
 			# Loop the shape matrix
 			STARTING_PIXEL = (4, 4)
@@ -245,11 +258,7 @@ def generate_page_font(config: dict, name: str, page_font: str, craft: dict|None
 						if not os.path.exists(image_path):
 							error(f"Missing item texture at '{image_path}'")
 						item_texture = Image.open(image_path)
-						factor = SQUARE_SIZE / item_texture.size[0]
-						item_texture = item_texture.resize(
-							(int(item_texture.size[0]*factor), int(item_texture.size[1]*factor)),
-							Image.NEAREST
-						)
+						item_texture = careful_resize(item_texture, SQUARE_SIZE)
 						coords = (
 							j * (SQUARE_SIZE + CASE_OFFSETS[0]) + STARTING_PIXEL[0],
 							i * (SQUARE_SIZE + CASE_OFFSETS[1]) + STARTING_PIXEL[1]
@@ -267,14 +276,16 @@ def generate_page_font(config: dict, name: str, page_font: str, craft: dict|None
 				template.paste(count_img, [x + 2 for x in coords], count_img)
 
 			# Save the image
-			template.save(f"{config['manual_path']}/font/page/{output_filename}.png")
+			if not config['manual_high_resolution']:
+				template.save(f"{config['manual_path']}/font/page/{output_filename}.png")
 
 		# Smelting craft
 		elif craft["type"] in FURNACES_RECIPES_TYPES:
 			
 			# Get the image template and append the provider
 			template = Image.open(f"{TEMPLATES_PATH}/furnace.png")
-			font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/page/{output_filename}.png", "ascent": 0 if not output_name else 6, "height": 60, "chars": [page_font]})
+			if not config['manual_high_resolution']:
+				font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/page/{output_filename}.png", "ascent": 0 if not output_name else 6, "height": 60, "chars": [page_font]})
 
 			# Place input item
 			input_item = ingr_to_id(craft["ingredient"])
@@ -283,11 +294,7 @@ def generate_page_font(config: dict, name: str, page_font: str, craft: dict|None
 			if not os.path.exists(image_path):
 				error(f"Missing item texture at '{image_path}'")
 			item_texture = Image.open(image_path)
-			factor = SQUARE_SIZE / item_texture.size[0]
-			item_texture = item_texture.resize(
-				(int(item_texture.size[0]*factor), int(item_texture.size[1]*factor)),
-				Image.NEAREST
-			)
+			item_texture = careful_resize(item_texture, SQUARE_SIZE)
 			mask = item_texture.convert("RGBA").split()[3]
 			template.paste(item_texture, (4, 4), mask)
 
@@ -299,16 +306,27 @@ def generate_page_font(config: dict, name: str, page_font: str, craft: dict|None
 				template.paste(count_img, [x + 2 for x in coords], count_img)
 			
 			# Save the image
-			template.save(f"{config['manual_path']}/font/page/{output_filename}.png")
+			if not config['manual_high_resolution']:
+				template.save(f"{config['manual_path']}/font/page/{output_filename}.png")
 	
 	# Else, there is no craft, just put the item in a box
 	else:
 		# Get the image template and append the provider
 		template = Image.open(f"{TEMPLATES_PATH}/simple_case_no_border.png")
+		factor = 1
+		if config['manual_high_resolution']:
+			factor = 256 // template.size[0]
+			result_texture = careful_resize(result_texture, SQUARE_SIZE * factor)
+			template = careful_resize(template, 256)
+			result_mask = result_texture.convert("RGBA").split()[3]
+		else:
+			# Resize the texture and get the mask
+			result_texture = careful_resize(result_texture, SQUARE_SIZE)
+			result_mask = result_texture.convert("RGBA").split()[3]
 		font_providers.append({"type":"bitmap","file":f"{config['namespace']}:font/page/{output_filename}.png", "ascent": 0 if not output_name else 6, "height": 40, "chars": [page_font]})
 
 		# Place the result item
-		template.paste(result_texture, (2, 2), result_mask)
+		template.paste(result_texture, (2 * factor, 2 * factor), result_mask)
 		template = add_border(template, BORDER_COLOR, BORDER_SIZE, is_rectangle_shape = True)
 		template.save(f"{config['manual_path']}/font/page/{output_filename}.png")
 	return
@@ -364,8 +382,8 @@ def convert_shapeless_to_shaped(craft: dict) -> dict:
 
 
 # Convert ingredient to formatted JSON for book
-COMPONENTS_TO_IGNORE = NOT_COMPONENTS + ["custom_data", "count"]
-def get_item_component(config: dict, ingredient: dict|str, only_those_components: list[str] = None) -> dict:
+COMPONENTS_TO_IGNORE = NOT_COMPONENTS + ["custom_data", "count", "profile", "dyed_color", "container"]
+def get_item_component(config: dict, ingredient: dict|str, only_those_components: list[str] = None, count: int = 1) -> dict:
 	""" Generate item hover text for a craft ingredient
 	Args:
 		ingredient (dict|str): The ingredient
@@ -387,7 +405,7 @@ def get_item_component(config: dict, ingredient: dict|str, only_those_components
 			id = ingredient
 			item = config['database'][ingredient]
 		else:
-			custom_data = ingredient["components"]["custom_data"]
+			custom_data: dict = ingredient["components"]["custom_data"]
 			id = ingr_to_id(ingredient, add_namespace = False)
 			if custom_data.get(config['namespace']):
 				item = config['database'].get(id)
@@ -417,6 +435,10 @@ def get_item_component(config: dict, ingredient: dict|str, only_those_components
 		if page_number != -1:
 			formatted["clickEvent"] = {"action":"change_page","value":str(page_number)}
 	
+	# High resolution
+	if config['manual_high_resolution']:
+		formatted["text"] = high_res_font_from_ingredient(config, ingredient, count)
+
 	# Return
 	return formatted
 
@@ -433,42 +455,53 @@ def generate_craft_content(config: dict, craft: dict, name: str, page_font: str)
 	"""
 	craft_type = craft["type"]
 	content = [{"text": "", "font": config['namespace'] + ':' + FONT_FILE, "color": "white"}]	# Make default font for every next component
-	
-	# Show up item title and page font
-	titled = name.replace("_", " ").title() + "\n"
-	content.append({"text": titled, "font": "minecraft:default", "color": "black", "underlined": True})
-	content.append(SMALL_NONE_FONT + page_font + "\n")
 
 	# Convert shapeless crafting to shaped crafting
 	if craft_type == "crafting_shapeless":
 		craft = convert_shapeless_to_shaped(craft)
 		craft_type = "crafting_shaped"
 	
+	# If high resolution, get proper page font
+	if config['manual_high_resolution']:
+		page_font = high_res_font_from_craft(craft)
+	
+	# Show up item title and page font
+	titled = name.replace("_", " ").title() + "\n"
+	content.append({"text": titled, "font": "minecraft:default", "color": "black", "underlined": True})
+	content.append(SMALL_NONE_FONT + page_font + "\n")
+	
 	# Generate the image for the page
 	generate_page_font(config, name, page_font, craft)
 
 	# Get result component
-	result_component = get_item_component(config, name)
+	result_count = craft.get("result_count", 1)
+	result_component = get_item_component(config, name, count = result_count)
 	if result_component.get("clickEvent"):
 		del result_component["clickEvent"]	# Remove clickEvent for result item (as we already are on the page)
+	result_component["text"] = MICRO_NONE_FONT + result_component["text"]	# Left adjustment
 
 	# If the craft is shaped
 	if craft_type == "crafting_shaped":
 
 		# Convert each ingredients to its text component
-		formatted_ingredients = {}
+		formatted_ingredients: dict[str, dict] = {}
 		for k, v in craft["ingredients"].items():
 			formatted_ingredients[k] = get_item_component(config, v)
 
 		# Add each ingredient to the craft
 		for line in craft["shape"]:
-			for _ in range(2):	# We need two lines to make a square, otherwise it will be a rectangle
+			for i in range(2):	# We need two lines to make a square, otherwise it will be a rectangle
 				content.append(SMALL_NONE_FONT)
 				for k in line:
 					if k == " ":
-						content.append(NONE_FONT)
+						content.append(INVISIBLE_ITEM_WIDTH)
 					else:
-						content.append(formatted_ingredients[k])
+						if i == 0:
+							content.append(formatted_ingredients[k])
+						else:
+							copy = formatted_ingredients[k].copy()
+							copy["text"] = INVISIBLE_ITEM_WIDTH
+							content.append(copy)
 				content.append("\n")
 		if len(craft["shape"]) == 1 and len(craft["shape"][0]) < 3:
 			content.append("\n")
@@ -481,7 +514,7 @@ def generate_craft_content(config: dict, craft: dict, name: str, page_font: str)
 			len_1 = len(craft["shape"][0])
 			offset_1 = 3 - len_1
 			break_line_pos = content.index("\n", content.index("\n") + 1)	# Find the second line break
-			content.insert(break_line_pos, NONE_FONT * offset_1)
+			content.insert(break_line_pos, (INVISIBLE_ITEM_WIDTH * offset_1))
 			content.insert(break_line_pos + 1, result_component)
 			
 			# Second layer of the square
@@ -490,8 +523,10 @@ def generate_craft_content(config: dict, craft: dict, name: str, page_font: str)
 			if len_2 == 0:
 				content.insert(break_line_pos + 2, "\n" + SMALL_NONE_FONT)
 			break_line_pos = content.index("\n", break_line_pos + 3)	# Find the third line break
-			content.insert(break_line_pos, NONE_FONT * offset_2)
-			content.insert(break_line_pos + 1, result_component)
+			content.insert(break_line_pos, (INVISIBLE_ITEM_WIDTH * offset_2))
+			copy = result_component.copy()
+			copy["text"] = INVISIBLE_ITEM_WIDTH
+			content.insert(break_line_pos + 1, copy)
 		else:
 			# First layer of the square
 			len_line = len(craft["shape"][1]) if len(craft["shape"]) > 1 else 0
@@ -502,7 +537,7 @@ def generate_craft_content(config: dict, craft: dict, name: str, page_font: str)
 			except:
 				content.append(SMALL_NONE_FONT)
 				break_line_pos = len(content)
-			content.insert(break_line_pos, NONE_FONT * (offset - 1) + SMALL_NONE_FONT * 2)
+			content.insert(break_line_pos, (INVISIBLE_ITEM_WIDTH * (offset - 1) + SMALL_NONE_FONT * 2))
 			content.insert(break_line_pos + 1, result_component)
 
 			# Second layer of the square
@@ -511,8 +546,10 @@ def generate_craft_content(config: dict, craft: dict, name: str, page_font: str)
 			except:
 				content.append("\n" + SMALL_NONE_FONT)
 				break_line_pos = len(content)
-			content.insert(break_line_pos, NONE_FONT * (offset - 1) + SMALL_NONE_FONT * 2)
-			content.insert(break_line_pos + 1, result_component)
+			content.insert(break_line_pos, (INVISIBLE_ITEM_WIDTH * (offset - 1) + SMALL_NONE_FONT * 2))
+			copy = result_component.copy()
+			copy["text"] = INVISIBLE_ITEM_WIDTH
+			content.insert(break_line_pos + 1, copy)
 
 			# Add break lines for the third layer of a 3x3 craft
 			if len(craft["shape"]) < 3 and len(craft["shape"][0]) == 3:
@@ -528,15 +565,25 @@ def generate_craft_content(config: dict, craft: dict, name: str, page_font: str)
 		formatted_ingredient = get_item_component(config, craft["ingredient"])
 
 		# Add the ingredient to the craft
-		for _ in range(2):
+		for i in range(2):
 			content.append(SMALL_NONE_FONT)
-			content.append(formatted_ingredient)
+			if i == 0:
+				content.append(formatted_ingredient)
+			else:
+				copy = formatted_ingredient.copy()
+				copy["text"] = INVISIBLE_ITEM_WIDTH
+				content.append(copy)
 			content.append("\n")
 		
 		# Add the result to the craft
-		for _ in range(2):
-			content.append(SMALL_NONE_FONT * 4 + NONE_FONT * 2)
-			content.append(result_component)
+		for i in range(2):
+			content.append(SMALL_NONE_FONT * 4 + INVISIBLE_ITEM_WIDTH * 2)
+			if i == 0:
+				content.append(result_component)
+			else:
+				copy = result_component.copy()
+				copy["text"] = INVISIBLE_ITEM_WIDTH
+				content.append(copy)
 			content.append("\n")
 		content.append("\n\n")
 
@@ -571,6 +618,7 @@ def generate_otherside_crafts(config: dict, item: str) -> list[dict]:
 	for key, value in config['database'].items():
 		if key != item and value.get(RESULT_OF_CRAFTING):
 			for craft in value[RESULT_OF_CRAFTING]:
+				craft: dict = craft
 				if ("ingredient" in craft and item == ingr_to_id(craft["ingredient"], False)) or \
 					("ingredients" in craft and isinstance(craft["ingredients"], dict) and item in [ingr_to_id(x, False) for x in craft["ingredients"].values()]) or \
 					("ingredients" in craft and isinstance(craft["ingredients"], list) and item in [ingr_to_id(x, False) for x in craft["ingredients"]]):
@@ -606,19 +654,22 @@ def generate_wiki_font_for_ingr(config: dict, name: str, craft: dict) -> str:
 		texture_path = f"{config['manual_path']}/items/{result_item}.png"
 		result_item = result_item.replace("/", "_")
 		dest_path = f"{config['manual_path']}/font/wiki_icons/{result_item}.png"
-		if not os.path.exists(dest_path):
 
-			# Load texture and resize it
-			item_texture = Image.open(texture_path)
-			item_texture = item_texture.resize((42, 42), Image.NEAREST)
-			item_texture = item_texture.convert("RGBA")
+		# Load texture and resize it
+		item_texture = Image.open(texture_path)
+		item_res = 64 if not config["manual_high_resolution"] else 256
+		item_res_adjusted = int(item_res * 0.75)
+		item_texture = careful_resize(item_texture, item_res_adjusted)
+		item_texture = item_texture.convert("RGBA")
 
-			# Load the template and paste the texture on it
-			template = Image.open(f"{TEMPLATES_PATH}/wiki_ingredient_of_craft_template.png")
-			template.paste(item_texture, (11, 11), item_texture)
-			
-			# Save the result
-			template.save(dest_path)
+		# Load the template and paste the texture on it
+		template = Image.open(f"{TEMPLATES_PATH}/wiki_ingredient_of_craft_template.png")
+		template = careful_resize(template, item_res)
+		offset = (item_res - item_res_adjusted) // 2
+		template.paste(item_texture, (offset, offset), item_texture)
+
+		# Save the result
+		template.save(dest_path)
 
 		# Prepare provider
 		font = get_next_font()
@@ -631,3 +682,86 @@ def generate_wiki_font_for_ingr(config: dict, name: str, craft: dict) -> str:
 	# Return the font
 	return font
 
+# Generate high res image for item
+def generate_high_res_font(config: dict, item: str, item_image: Image.Image, count: int = 1) -> str:
+	""" Generate the high res font to display in the manual for the item
+	Args:
+		item		(str):		The name of the item, ex: "adamantium_fragment"
+		item_image	(Image):	The image of the item
+		count		(int):		The count of the item
+	Returns:
+		str: The font to the generated texture
+	"""
+	font = get_next_font()
+	item = f"{item}_{count}" if count > 1 else item
+	
+	# Get output path
+	path = f"{config['manual_path']}/font/high_res/{item}.png"
+	provider_path = f"{config['namespace']}:font/high_res/{item}.png"
+	for p in font_providers:	# Check if it already exists
+		if p["file"] == provider_path:
+			return MICRO_NONE_FONT + p["chars"][0]
+	font_providers.append({"type":"bitmap","file": provider_path, "ascent": 7, "height": 16, "chars": [font]})
+
+
+	# Generate high res font
+	os.makedirs(os.path.dirname(path), exist_ok = True)
+	high_res: int = 256
+	resized = careful_resize(item_image, high_res)
+	resized = resized.convert("RGBA")
+
+	# Add the item count
+	if count > 1:
+		img_count = image_count(count)
+		img_count = careful_resize(img_count, high_res)
+		resized.paste(img_count, (0, 0), img_count)
+
+	# Add invisible pixels for minecraft font at each corner
+	total_width = resized.size[0] - 1
+	total_height = resized.size[1] - 1
+	angles = [(0, 0), (total_width, 0), (0, total_height), (total_width, total_height)]
+	for angle in angles:
+		resized.putpixel(angle, (0, 0, 0, 100))
+
+	# Save the result and return the font
+	resized.save(path)
+	return MICRO_NONE_FONT + font
+
+# Call the previous function
+def high_res_font_from_ingredient(config: dict, ingredient: str|dict, count: int = 1) -> str:
+	""" Generate the high res font to display in the manual for the ingredient
+	Args:
+		ingredient	(str|dict):	The ingredient, ex: "adamantium_fragment" or {"item": "minecraft:stick"} or {"components": {"custom_data": {"iyc": {"adamantium_fragment": true}}}}
+		count		(int):		The count of the item
+	Returns:
+		str: The font to the generated texture
+	"""
+	# Decode the ingredient
+	if isinstance(ingredient, dict):
+		ingredient = ingr_to_id(ingredient, add_namespace = True)
+	if ':' in ingredient:
+		image_path = f"{config['manual_path']}/items/{ingredient.replace(':', '/')}.png"
+		if not os.path.exists(image_path):
+			error(f"Missing item texture at '{image_path}'")
+		item_image = Image.open(image_path)
+		ingredient = ingredient.split(":")[1]
+	else:
+		item_image = Image.open(f"{config['manual_path']}/items/{config['namespace']}/{ingredient}.png")
+	
+	# Generate the high res font
+	return generate_high_res_font(config, ingredient, item_image, count)
+
+# Util function
+@simple_cache
+def high_res_font_from_craft(craft: dict) -> str:
+	if craft["type"] in FURNACES_RECIPES_TYPES:
+		return FURNACE_FONT
+	elif craft["type"] == "crafting_shaped":
+		if len(craft["shape"]) == 3 or len(craft["shape"][0]) == 3:
+			return SHAPED_3X3_FONT
+		else:
+			return SHAPED_2X2_FONT
+	else:
+		error(f"Unknown craft type '{craft['type']}'")
+		return ""
+
